﻿using MBN;
using System.Threading;
using MBN.Exceptions;
using MBN.Modules;
using Microsoft.SPOT;

namespace Examples
{
    public class Program
    {
        private static GPS3Click gps;

        public static void Main()
        {
            try
            {
                gps = new GPS3Click(Hardware.SocketTwo);
            }
            catch (DeviceInitialisationException ex)
            {
                Debug.Print(ex.Message);
                throw new DeviceInitialisationException("Unable to initialize GPS3 Click");
            }

            gps.PositionChange += GpsPositionChange;
            gps.AntennaStatusChanged += gps_AntennaStatusChanged;
            gps.NMEASentenceReceived += gps_NMEASentenceReceived;
            gps.PMTKSentenceReceived += gps_PMTKSentenceReceived;
            gps.SerialError += gps_SerialError;
            gps.SpeedChanged += GpsSpeedChanged;
            gps.CoordinatesChanged += GpsCoordinatesChanged;
            gps.CourseChanged += GpsCourseChanged;
            gps.PositioningModeChanged += GpsPositioningModeChanged;

            gps.StartGPS();

            Thread.Sleep(Timeout.Infinite);
        }

        static void GpsPositioningModeChanged(GPS3Click sender, GPS3Click.PositioningModes positioningMode)
        {
            Debug.Print("RMC Positioning Mode changed to  " + (positioningMode == GPS3Click.PositioningModes.NoFix ? "No Fix" : positioningMode == GPS3Click.PositioningModes.AutonomousFix ? "Autonomous Fix" : "Differential Fix") + "\n");
        }

        static void GpsCourseChanged(GPS3Click sender, double heading)
        {
            Debug.Print("Course changed to  - " + heading);
        }

        static void GpsCoordinatesChanged(GPS3Click sender, double latitude, double longitude)
        {
            Debug.Print("Coordinates changed to Longitude - " + longitude + ", Latitude - " + latitude);
        }

        static void GpsSpeedChanged(GPS3Click sender, double knots)
        {
            Debug.Print("Speed changed to - " + knots + "(Knots)");
        }

        static void GpsPositionChange(GPS3Click sender, GPS3Click.Position position)
        {
            if (!position.DataValid) return;
            Debug.Print(position.ToString());
        }

        static void gps_SerialError(object sender, string errorDescription, System.DateTime eventTime)
        {
            Debug.Print("Serial error - " + errorDescription + "\n");
            gps.Serial.DiscardInBuffer();
            gps.Serial.Flush();
        }

        static void gps_PMTKSentenceReceived(GPS3Click sender, string pmtksentence)
        {
            Debug.Print("PMTK Sentence - " + pmtksentence);
        }

        static void gps_AntennaStatusChanged(GPS3Click sender, GPS3Click.Antenna status)
        {
            Debug.Print("Antenna Status changed to - " + (status == GPS3Click.Antenna.Ok ? "Open" : status == GPS3Click.Antenna.Ok ? "Ok" : "Shorted"));
        }

        static void gps_NMEASentenceReceived(GPS3Click sender, string nmeasentence)
        {
            Debug.Print("NMEA Sentence - " + nmeasentence);
        }
    }
}
